import { Injectable } from '@angular/core';
import { Http, Headers, RequestOptions, Response } from '@angular/http';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/observable/of';
import { environment } from '../environments/environment';
import { UserProfileService } from './user-profile.service';
import * as auth0 from 'auth0-js';


@Injectable()
export class AuthService {
  // authUrl = environment.url + 'auth.php';
  // headers                = new Headers({'Content-Type': 'application/json'});
  // options                = new RequestOptions({headers: this.headers/*, withCredentials: true*/});

  userinfo: any;

  auth0 = new auth0.WebAuth({
    clientID: 'Dx8R1paOZzhRhTdpXvTkZd01Ie9TwniO',
    domain: 'tzuchien.auth0.com',
    responseType: 'token id_token',
    audience: 'https://tzuchien.auth0.com/userinfo',
    redirectUri: 'http://localhost:4200/',
    scope: 'openid profile email'


  });

  constructor(private http: Http) { }

  login() {
    this.auth0.authorize();
    //return Observable.of(true);
  }
  //處理oauth認證後資料處理
  handleAuthentication() {
    //通過auth0.parseHash取得authResult相關資料
    this.auth0.parseHash((err, authResult) => {
      if (authResult && authResult.accessToken && authResult.idToken) {
        //再透過auth0.client.userinfoauth(Result.accessToken)取得user資料
        this.auth0.client.userInfo(authResult.accessToken, (err, profile) => {
          //檢驗是否有取得profile
          console.log(profile.email);
          this.checkDBUser(profile)
            .subscribe((data: any) => {
              //console.log('user data: ' + data.user.email);

              if (!data.user) {
                //如果db不存在此user
                //將Auth0提供的user profile存到DB
                this.createUser(profile)
                  .subscribe(
                  data => {
                    this.setSession(authResult);
                    //db回傳的存到localStorage
                    localStorage.setItem('profile', JSON.stringify(data.savedUser));
                    this.userinfo = data.savedUser;
                  },
                  err => {
                    console.log(err);
                    this.logout();
                  });
              } else {
                console.log('User already in DB,load user data');
                this.loadUser(profile)
                  .subscribe(data => {
                    console.log('LoadedUser:', data.loadedUser.email);
                    // 把載入的user存到localStorage
                    this.setSession(authResult);
                    localStorage.setItem('profile', JSON.stringify(data));
                    this.userinfo = data.loadedUser;
                  });
              }
            });
        });
        window.location.hash = '';
      } else if (err) {
        console.log(err);
      }

    });
  }

  setSession(authResult) {
    const expiresAt = JSON.stringify((authResult.expiresIn * 1000) + new Date().getTime());
    localStorage.setItem('access_token', authResult.accessToken);
    localStorage.setItem('id_token', authResult.idToken);
    localStorage.setItem('expires_at', expiresAt);
  }
  public isAuthenticated(): boolean {
    const expiresAt = JSON.parse(localStorage.getItem('expires_at'));
    return new Date().getTime() < expiresAt;

  }

  //Check User if exist
  checkDBUser(profile) {
    return this.http.get(`http://localhost:3000/api/checkDBUser/${profile.email}`)
      .map(res => res.json());
  }
  //User exist
  loadUser(profile) {
    console.log("load user... ");
    return this.http.get(`http://localhost:3000/api/user/${profile.email}`)
      .map(res => res.json());
  }
  //add user
  createUser(profile) {
    console.log("create user... ");
    //加上header info
    let headers = new Headers({ 'Content-Type': 'application/json' });
    let options = new RequestOptions({ headers: headers })
    let bodyString = JSON.stringify(profile);

    //body轉成string且headers使用物件
    return this.http.post('http://localhost:3000/api/user/', bodyString, options)
      .map(res => res.json());

  }
  //登出
  logout() {
    localStorage.removeItem('access_token');
    localStorage.removeItem('id_token');
    localStorage.removeItem('expires_at');
  }

}
