import { Component, OnInit, OnDestroy } from '@angular/core';
import { TodoService } from '../todo.service';
import { Subscription } from 'rxjs/Subscription';
import {Router} from "@angular/router";

@Component({
  selector: 'app-todo-list',
  templateUrl: './todo-list.component.html',
  styleUrls: ['./todo-list.component.css']
})
export class TodoListComponent implements OnInit, OnDestroy {

  todos: any[] = [];
  subscription: Subscription;
  loading: boolean = false;
  result = null;

  constructor(
      private todoService: TodoService,
      private router: Router
  ) { }

  deleteTodo(id) {
    // console.log("delete " + id);
    this.todoService
        .deleteTodo(id)
        .subscribe(() => this.getTodos());
  }

  newTodo() {
    this.router.navigate(['/todos/new']);
  }

  toggleDone(id) {
    this.todoService.toggleDone(id)
        .subscribe(
            () => this.todos.map(todo => todo.done = todo._id === id ? !todo.done : todo.done)
        );
  }

  ngOnInit() {
    this.getTodos();
  }

  ngOnDestroy() {
    this.subscription.unsubscribe();
  }

  private getTodos() {
    this.loading = true;
    this.subscription = this.todoService.getTodos(1000)
                            .subscribe(
                                todos => {
                                  this.todos = todos;
                                  this.loading = false;
                                }
                            );
  }

}
